package api

import (
	"fmt"
	"log"
	"net/http"
	"strconv"

	"github.com/gin-gonic/gin"

	"git.nemunai.re/eurocollec/struct"
)

func declareCoinsRoutes(router *gin.RouterGroup) {
	router.GET("/coins", func(c *gin.Context) {
		var coins []*euroc.Coin
		var err error

		if cntry, ok := c.Get("country"); ok {
			country := cntry.(*euroc.Country)
			coins, err = country.GetCoins()
		} else {
			coins, err = euroc.GetCoins()
		}

		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}
		c.JSON(http.StatusOK, coins)
	})
	router.POST("/coins", newCoin)
	router.DELETE("/coins", func(c *gin.Context) {
		_, err := euroc.ClearCoins()
		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}
		c.JSON(http.StatusOK, true)
	})

	apiCoinsRoutes := router.Group("/coins/:cid")
	apiCoinsRoutes.Use(coinHandler)

	apiCoinsRoutes.GET("", func(c *gin.Context) {
		coin := c.MustGet("coin").(*euroc.Coin)
		c.JSON(http.StatusOK, coin)
	})
	apiCoinsRoutes.PUT("", updateCoin)
	apiCoinsRoutes.DELETE("", func(c *gin.Context) {
		coin := c.MustGet("coin").(*euroc.Coin)
		_, err := coin.Delete()
		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return

		}

		c.JSON(http.StatusOK, true)
	})

	declareCoinsHaveRoutes(apiCoinsRoutes)
}

func coinHandler(c *gin.Context) {
	if cid, err := strconv.ParseInt(c.Param("cid"), 10, 64); err != nil {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": "Not a valid coin identifier"})
	} else if coin, err := euroc.GetCoin(cid); err != nil {
		c.AbortWithStatusJSON(http.StatusNotFound, gin.H{"errmsg": "Coin not found"})
	} else {
		c.Set("coin", coin)
		c.Next()
	}
}

func newCoin(c *gin.Context) {
	var uc euroc.Coin
	err := c.ShouldBindJSON(&uc)
	if err != nil {
		log.Printf("%s sends invalid Coin JSON: %w", c.ClientIP(), err)
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": fmt.Sprintf("Something is wrong in received data: %s", err.Error())})
		return
	}

	if len(uc.Label) == 0 {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": "Coin's label cannot be empty"})
		return
	}

	var l *euroc.Country
	if country, ok := c.Get("country"); ok {
		l = country.(*euroc.Country)
	} else if uc.IdCountry != nil {
		if country, err := euroc.GetCountry(*uc.IdCountry); err != nil {
			c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": err.Error()})
			return
		} else {
			l = country
		}
	}

	coin, err := euroc.NewCoin(l, nil, uc.Label, uc.Year, uc.Icon, uc.Description, uc.Engraver, uc.Copies, uc.Slice)
	if err != nil {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": err.Error()})
		return
	}

	c.JSON(http.StatusOK, coin)
}

func updateCoin(c *gin.Context) {
	coin := c.MustGet("coin").(*euroc.Coin)

	var uc euroc.Coin
	err := c.ShouldBindJSON(&uc)
	if err != nil {
		log.Printf("%s sends invalid Coin JSON: %w", c.ClientIP(), err)
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": fmt.Sprintf("Something is wrong in received data: %s", err.Error())})
		return
	}

	if len(uc.Label) == 0 {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": "Coin's label cannot be empty"})
		return
	}

	uc.Id = coin.Id

	if _, err := uc.Update(); err != nil {
		c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
		return
	}

	c.JSON(http.StatusOK, uc)
}
