package api

import (
	"fmt"
	"log"
	"net/http"
	"strconv"

	"github.com/gin-gonic/gin"

	"git.nemunai.re/eurocollec/struct"
)

func declareCountriesRoutes(router *gin.RouterGroup) {
	router.GET("/countries", func(c *gin.Context) {
		cntrs, err := euroc.GetCountries()
		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}

		c.JSON(http.StatusOK, cntrs)
	})
	router.POST("/countries", newCountry)
	router.DELETE("/countries", func(c *gin.Context) {
		ret, err := euroc.ClearCountries()
		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}

		c.JSON(http.StatusOK, ret)
	})

	apiCountriesRoutes := router.Group("/countries/:lid")
	apiCountriesRoutes.Use(countryHandler)

	apiCountriesRoutes.GET("", func(c *gin.Context) {
		country := c.MustGet("country").(*euroc.Country)
		c.JSON(http.StatusOK, country)
	})
	apiCountriesRoutes.PUT("", updateCountry)
	apiCountriesRoutes.DELETE("", func(c *gin.Context) {
		country := c.MustGet("country").(*euroc.Country)
		ret, err := country.Delete()

		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}

		c.JSON(http.StatusOK, ret)
	})

	apiCountriesRoutes.GET("stats", countryStats)

	declareCoinsRoutes(apiCountriesRoutes)
}

func countryHandler(c *gin.Context) {
	if lid, err := strconv.ParseInt(c.Param("lid"), 10, 64); err != nil {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": "Not a valid country identifier"})
	} else if country, err := euroc.GetCountry(lid); err != nil {
		c.AbortWithStatusJSON(http.StatusNotFound, gin.H{"errmsg": "Country not found"})
	} else {
		c.Set("country", country)
		c.Next()
	}
}

func newCountry(c *gin.Context) {
	var uc euroc.Country
	err := c.ShouldBindJSON(&uc)
	if err != nil {
		log.Printf("%s sends invalid Country JSON: %w", c.ClientIP(), err)
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": fmt.Sprintf("Something is wrong in received data: %s", err.Error())})
		return
	}

	if len(uc.Label) == 0 {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": "Country's label cannot be empty"})
		return
	}

	cntry, err := euroc.NewCountry(uc.Label, uc.Icon, uc.Since)
	if err != nil {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": err.Error()})
		return
	}

	c.JSON(http.StatusOK, cntry)
}

func updateCountry(c *gin.Context) {
	country := c.MustGet("country").(*euroc.Country)

	var uc euroc.Country
	err := c.ShouldBindJSON(&uc)
	if err != nil {
		log.Printf("%s sends invalid Country JSON: %w", c.ClientIP(), err)
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": fmt.Sprintf("Something is wrong in received data: %s", err.Error())})
		return
	}

	if len(uc.Label) == 0 {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": "Country's label cannot be empty"})
		return
	}

	uc.Id = country.Id

	if _, err := uc.Update(); err != nil {
		c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
		return
	}

	c.JSON(http.StatusOK, uc)
}
