package api

import (
	"fmt"
	"log"
	"net/http"
	"strconv"

	"github.com/gin-gonic/gin"

	"git.nemunai.re/eurocollec/struct"
)

func declareCoinsHaveRoutes(router *gin.RouterGroup) {
	router.GET("/have", func(c *gin.Context) {
		coin := c.MustGet("coin").(*euroc.Coin)
		haves, err := coin.GetHaves()
		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}
		c.JSON(http.StatusOK, haves)
	})
	router.POST("/have", newHaveCoin)
	router.DELETE("/have", func(c *gin.Context) {
		coin := c.MustGet("coin").(*euroc.Coin)
		ret, err := coin.ClearHaves()
		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}
		c.JSON(http.StatusOK, ret)
	})

	apiCoinsHaveRoutes := router.Group("/have/:hid")
	apiCoinsHaveRoutes.Use(haveHandler)

	apiCoinsHaveRoutes.GET("", func(c *gin.Context) {
		have := c.MustGet("have").(*euroc.HaveCoin)
		c.JSON(http.StatusOK, have)
	})
	apiCoinsHaveRoutes.PUT("", updateHaveCoin)
	apiCoinsHaveRoutes.DELETE("", func(c *gin.Context) {
		have := c.MustGet("have").(*euroc.HaveCoin)
		ret, err := have.Delete()
		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}
		c.JSON(http.StatusOK, ret)
	})
}

func haveHandler(c *gin.Context) {
	coin := c.MustGet("coin").(*euroc.Coin)

	if hid, err := strconv.ParseInt(c.Param("hid"), 10, 64); err != nil {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": "Not a valid coin's have identifier"})
	} else if have, err := coin.GetHave(hid); err != nil {
		c.AbortWithStatusJSON(http.StatusNotFound, gin.H{"errmsg": "Coin's have not found"})
	} else {
		c.Set("have", have)
		c.Next()
	}
}

func newHaveCoin(c *gin.Context) {
	coin := c.MustGet("coin").(*euroc.Coin)

	var uh euroc.HaveCoin
	err := c.ShouldBindJSON(&uh)
	if err != nil {
		log.Printf("%s sends invalid Coin's have JSON: %w", c.ClientIP(), err)
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": fmt.Sprintf("Something is wrong in received data: %w", err)})
		return
	}

	if len(uh.Cleaness) == 0 {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": "Coin's cleaness cannot be empty"})
		return
	}

	h, err := coin.NewHave(uh.Cleaness, uh.Comment)
	if err != nil {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": err.Error()})
		return
	}

	c.JSON(http.StatusOK, h)
}

func updateHaveCoin(c *gin.Context) {
	have := c.MustGet("have").(*euroc.HaveCoin)

	var uh euroc.HaveCoin
	err := c.ShouldBindJSON(&uh)
	if err != nil {
		log.Printf("%s sends invalid Coin's have JSON: %w", c.ClientIP(), err)
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": fmt.Sprintf("Something is wrong in received data: %w", err)})
		return
	}

	if len(uh.Cleaness) == 0 {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": err.Error()})
		return
	}

	uh.Id = have.Id

	if _, err := uh.Update(); err != nil {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": err.Error()})
		return
	}

	c.JSON(http.StatusOK, uh)
}
